\name{AnnotationDbi internals}

\alias{debugSQL}

\alias{class:FlatBimap}
\alias{FlatBimap-class}
\alias{FlatBimap}

\alias{class:AnnObj}
\alias{AnnObj-class}
\alias{AnnObj}

\alias{showQCData}

\alias{class:L2Rlink}
\alias{L2Rlink-class}
\alias{L2Rlink}

\alias{initialize,FlatBimap-method}
\alias{initialize,L2Rlink-method}
\alias{show,L2Rlink-method}

\alias{dbGetTable}
\alias{dbFileConnect}
\alias{dbFileDisconnect}
\alias{mergeToNamespaceAndExport}


\alias{createAnnObjs.SchemaChoice}
\alias{NCBICHIP_DB_SeedGenerator}
\alias{getOrgPkgForSchema}
\alias{filterSeeds}
\alias{chooseCentralOrgPkgSymbol}
\alias{mapIds_base}
\alias{resort_base}
\alias{testForValidKeytype}
\alias{testSelectArgs}


\title{AnnotationDbi internals}

\description{
  AnnotationDbi objects, classes and methods that are not intended to
  be used directly.
}

\keyword{internal}
\keyword{methods}
\keyword{classes}
